--------------------------------------------------------------------------
-- Babelfish.lua - Created by Jackson Howard Colorado, free public use
--------------------------------------------------------------------------
-- 01/16/05 [jhc] Created as proof of concept.  Has encode, translate, and
--		  autotranslate commands.  I welcome any input!


function babelfish_initialize()
--------------------------------------------------------------------
-- Purpose: Initializes the Babelfish commands on load.
-- Accepts: N/A
-- Assumes: That it is being run at load time.
-- Returns: N/A
-- Affects: Creates "speakbabel", "sb", "readbabel", and "rb" commands
--	    in the slash command list.
--------------------------------------------------------------------
	-- Make the "Speak in Babel" command.
	SlashCmdList["BABEL"] = speakbabel_command;
	SLASH_BABEL1 = "/speakbabel";
	SLASH_BABEL2 = "/sb";
	-- Make the "Yell in Babel" command.
	SlashCmdList["YELLBABEL"] = yellbabel_command;
	SLASH_YELLBABEL1 = "/yellbabel";
	SLASH_YELLBABEL2 = "/yb";
	-- Make the "Translate (Read) Babel" command.
	SlashCmdList["READBABEL"] = readbabel_command;
	SLASH_READBABEL1 = "/readbabel";
	SLASH_READBABEL2 = "/rb"
	-- Send an active message.
	ChatFrame1:AddMessage("Babelfish is now active, awaiting translation.", 1.0, .5, 1.0)
end


function ReadBabel(msg)
--------------------------------------------------------------------
-- Purpose: Translates a given message from Babel to ASCII.
-- Accepts: msg (String) - The message to translate to ASCII.
-- Assumes: N/A
-- Returns: (String) The translated message.
--------------------------------------------------------------------
	-- First, we will need to translate the message, one character at a time,
	-- until it is translated in entirety.
	Output = ""
	CurAscii = ""
	if string.sub(msg, 1, 2)=="@#" then
		-- Loop until we find a "#", starting at the third character (the first two
		-- were identifying characters)
		for i = 3, string.len(msg) do
			-- If we've come to a #, use the current Ascii holder to add to output.
			if string.sub(msg, i, i)=="#" then
				-- Add a character to the output.
				Output = Output .. string.char(tonumber(CurAscii))
				-- Reset the Current Ascii holder.
				CurAscii = ""
			else
				-- Append this number character to the Ascii holder.
				CurAscii = CurAscii .. string.sub(msg, i, i)
			end
		end
	end
	return Output
end


function SpeakBabel(msg)
--------------------------------------------------------------------
-- Purpose: Translates a given message from ASCII to Babel.
-- Accepts: msg (String) - The message to translate to Babel.
-- Assumes: N/A
-- Returns: (String) The translated Babel message.
--------------------------------------------------------------------
	-- When we wish to speak a Babel statement, this will translate our
	-- english ASCII text into Babel gibberish.

	-- We will need to encode the message, one character at a time,
	-- until it is encoded in its entirety.
	-- Begin by creating the basic output.  "@#" means that the following
	-- is a translatable statement.
	Output = "@#"
	-- Loop through the message, and for every character add the ascii number
	-- to the output.
	for i = 1, string.len(msg) do
		Output = Output .. string.byte(msg, i) .. "#"
	end
	return Output
end


function speakbabel_command(msg)
--------------------------------------------------------------------
-- Purpose: Speaks a Babel line.
-- Accepts: msg (String) - A message to encode to Babel.
-- Assumes: N/A
-- Returns: N/A
-- Affects: Says the Babel message out loud in a "Say" statement.
--------------------------------------------------------------------
	-- Now speak the message.
	SendChatMessage(SpeakBabel(msg), "SAY")
end


function yellbabel_command(msg)
--------------------------------------------------------------------
-- Purpose: Yells a Babel line.
-- Accepts: msg (String) - A message to encode to Babel.
-- Assumes: N/A
-- Returns: N/A
-- Affects: Says the Babel message out loud in a "Say" statement.
--------------------------------------------------------------------

	-- yell the message.
	SendChatMessage(SpeakBabel(msg), "YELL")
end


function readbabel_command(msg)
--------------------------------------------------------------------
-- Purpose: Used when a user wants to manually translate a message, it
--	    calls ReadBabel to translate and displays the result.
-- Accepts: msg (String) - The message to translate to ASCII.
-- Assumes: N/A
-- Returns: N/A
-- Affects: prints a translated phrase to the chat window.
--------------------------------------------------------------------
	-- Translate the Babel.
	ReadOut = ReadBabel(msg)
	-- If it was correctly translated, display a translation message.
	if (ReadOut ~= "") then
		ChatFrame1:AddMessage("Translated: " .. ReadOut, 1.0, .5, 1.0)
	else
		-- Otherwise, it could not be translated.  Inform the user.
		ChatFrame1:AddMessage("Message was not in proper Babel, and could not be translated.", 1.0, .5, 1.0)
	end
end


function autotranslate(msg, author)
--------------------------------------------------------------------
-- Purpose: Run automatically when a Babel message appears, this translates
--	    the message using ReadBabel and displays it along with the speaker's name.
-- Accepts: msg (String) - The message to translate to ASCII.
--	    author (String) - The name of the person who spoke the message.
-- Assumes: N/A
-- Returns: N/A
-- Affects: prints a translated phrase to the chat window.
--------------------------------------------------------------------
	-- First, check if it's your own Babel.  No point in translating your own Babel.
	if (author ~= UnitName("player")) then
		-- Translate the Babel.
		ReadOut = ReadBabel(msg)
		-- If it was correctly translated, display a translation message.
		if (ReadOut ~= "") then
			ChatFrame1:AddMessage("[" .. author .. "] (Translated) " .. ReadOut, 1.0, .5, 1.0)
		else
			-- Otherwise, it was incorrect.  Display a message.
			ChatFrame1:AddMessage("[" .. author .. "] (Improper Babel spoken, could not be translated.)", 1.0, .5, 1.0)
		end
	end
end